from os import listdir
from os.path import isfile, join

from pygame import Color, Rect, Surface
from pygame.image import load
from pygame.transform import flip
from pygame.locals import *

from Animation import Animation
from Vector import Vector

class Sprite(Animation):

    def __init__(self, parent, framerate=None):
        Animation.__init__(self, parent, self.shift_frame, framerate)
        self.clear_frames()
        self.frame_index = 0
        self.mirrored = False
        self.rect = Rect((0, 0, 0, 0))
        self.motion_overflow = Vector()
        self.display_surface = self.get_display_surface()

    def set_framerate(self, framerate):
        self.register(self.shift_frame, interval=framerate)

    def load_from_path(self, path, transparency=False, ppa=True, key=None):
        if isfile(path):
            paths = [path]
        else:
            paths = [join(path, name) for name in sorted(listdir(path))]
        for path in paths:
            img = load(path)
            if transparency:
                if ppa:
                    frame = img.convert_alpha()
                else:
                    frame = self.fill_colorkey(img, key)
            else:
                frame = img.convert()
            self.add_frame(frame)

    def fill_colorkey(self, img, key=None):
        if not key:
            key = (255, 0, 255)
        img = img.convert_alpha()
        frame = Surface(img.get_size())
        frame.fill(key)
        frame.set_colorkey(key)
        frame.blit(img, (0, 0))
        return frame

    def add_frame(self, frame):
        self.frames.append(frame)
        self.measure_rect()
        if len(self.frames) > 1:
            self.play()

    def measure_rect(self):
        max_width, max_height = 0, 0
        for frame in self.frames:
            width, height = frame.get_size()
            max_width = max(width, max_width)
            max_height = max(height, max_height)
        self.rect.size = max_width, max_height

    def shift_frame(self):
        if len(self.frames) > 1:
            self.increment_frame_index()

    def get_current_frame(self):
        return self.frames[self.frame_index]

    def increment_frame_index(self):
        index = self.frame_index + 1
        if index >= len(self.frames):
            index = 0
        self.frame_index = index

    def move(self, dx=0, dy=0):
        rect = self.rect
        if isinstance(dx, float) or isinstance(dy, float):
            excess = self.update_motion_overflow(dx, dy)
            rect.move_ip(int(dx) + excess[0], int(dy) + excess[1])
        else:
            rect.move_ip(dx, dy)

    def update_motion_overflow(self, dx, dy):
        overflow = self.motion_overflow
        overflow.move(dx - int(dx), dy - int(dy))
        excess = map(int, overflow)
        overflow[0] -= int(overflow[0])
        overflow[1] -= int(overflow[1])
        return excess

    def reset_motion_overflow(self):
        self.motion_overflow.place_at_origin()

    def collide(self, other):
        rect = other
        if not isinstance(other, Rect):
            rect = other.rect
        return self.rect.colliderect(rect)

    def mirror(self):
        frames = self.frames
        for ii, frame in enumerate(frames):
             frames[ii] = flip(frame, True, False)
        self.mirrored = not self.mirrored

    def clear_frames(self):
        self.frames = []

    def update(self):
        Animation.update(self)
        self.draw()

    def draw(self):
        self.display_surface.blit(self.get_current_frame(), self.rect)
